from .download import download_file
from pathlib import Path
down_path = Path(__file__).parent
xz_url = "addons_file"+"/"+down_path.name+"/"
download_file(xz_url + "Higgsas_Nodes.blend",down_path)


bl_info = {
    "name" : "Higgsas Nodes(addon)",
    "author" : "wxz-repack",
    "description" : "A collection of useful nodegroups to enhance Geometry Nodes.",
    "blender" : (4, 2, 0),
    "version" : (4, 1, 6),
    "location" : "Geometry Nodes Editor > Add > Higgsas",
    "warning" : "Restart Blender after disabling/uninstalling",
    "category" : "Node",
    "tracker_url" : "http://higgsas-geo-nodes-manual.rtfd.io/"
}
import json
import os
import re
from pathlib import Path

import bpy
from bpy.props import (BoolProperty, EnumProperty, PointerProperty,
                       StringProperty)
from bpy.types import Menu, Operator




def add_hig_button(self, context):
    if context.area.ui_type == 'GeometryNodeTree':
        self.layout.menu('NODE_MT_Higgsas_menu', text="Higgsas Nodes", icon='EVENT_H')

geo_node_group_cache = {}
geo_cat_list = []

dir_path = os.path.dirname(__file__)

# adapted code from https://github.com/blender/blender/blob/master/release/scripts/modules/nodeitems_utils.py
def geo_cat_generator():
    global geo_cat_list
    geo_cat_list = []
    for item in geo_node_group_cache.items():
        def custom_draw(self,context):
            layout = self.layout
            for group_name in geo_node_group_cache[self.bl_label]:
                props = layout.operator(
                    NODE_OT_group_add.bl_idname,
                    text=re.sub(r'.*?_', '', group_name), # filter prefix
                )
                props.group_name = group_name

        menu_type = type("NODE_MT_category_" + item[0], (bpy.types.Menu,), {
            "bl_idname": "NODE_MT_category_" + item[0].replace(" ", "_"),   # replace whitespace with uderscore to avoid alpha-numeric suffix warning 
            "bl_space_type": 'NODE_EDITOR',
            "bl_label": item[0],
            "draw": custom_draw,
        })
        if menu_type not in geo_cat_list:
            def generate_menu_draw(name,label): # Wrapper function to force unique references
                def draw_menu(self,context):
                    self.layout.menu(name, text=label)
                return draw_menu
            bpy.utils.register_class(menu_type)
            bpy.types.NODE_MT_Higgsas_menu.append(generate_menu_draw(menu_type.bl_idname,menu_type.bl_label))
            geo_cat_list.append(menu_type)


class NODE_MT_Higgsas_menu(Menu):
    bl_label = "Higgsas Nodes"
    bl_idname = 'NODE_MT_Higgsas_menu'

    @classmethod
    def poll(cls,context):
        return context.space_data.tree_type == 'GeometryNodeTree'

    def draw(self, context):
        pass

class NODE_OT_group_add(Operator):
    """Add a node group"""
    bl_idname = "hig." + os.path.basename(dir_path).lower()
    bl_label = "Add node group"
    bl_description = "Append Node Group"
    bl_options = {'REGISTER', 'UNDO'}

    group_name: StringProperty()

    @classmethod
    def poll(cls,context):
        return context.space_data.node_tree

    def execute(self, context):
        old_groups = set(bpy.data.node_groups)
        
        for file in os.listdir(dir_path):
            if file.endswith(".blend"):
                filepath = os.path.join(dir_path,file)
                break
        else:
            raise FileNotFoundError("No .blend File in directory "+ dir_path)
                
        with bpy.data.libraries.load(filepath, link=False) as (data_from, data_to):
            if self.group_name not in bpy.data.node_groups:
                data_to.node_groups.append(self.group_name)
        added_groups = list(set(bpy.data.node_groups)-old_groups)
        for group in added_groups:
            for node in group.nodes:
                if node.type == "GROUP":
                    new_name = node.node_tree.name.split(".")[0]
                    node.node_tree = bpy.data.node_groups[new_name]
        for group in added_groups:
            if "." in group.name:
                bpy.data.node_groups.remove(group) 

        bpy.ops.node.add_node(type="GeometryNodeGroup")
            
        node = context.selected_nodes[0]
        node.node_tree = bpy.data.node_groups[self.group_name]
        bpy.ops.transform.translate('INVOKE_DEFAULT')
        
        return {'FINISHED'}

def search_prop_group_by_ntree(self,context):
        for prop in context.scene.use_render:
            if prop.n_tree == context.space_data.node_tree:
                return prop
            

class override_use_render(bpy.types.Operator):
    bl_idname = "hig.override_use_render"
    bl_label = "Override Show Render"
    bl_description = "Overrieds all 'Show Render' Checkboxes in the Current Node Tree"
    bl_options = {'REGISTER', 'UNDO'}

    def execute(self, context):
        space = context.space_data
        context.scene.use_render.add().n_tree = space.node_tree
        return {'FINISHED'}

class NODE_PT_HIG_Options(bpy.types.Panel):
    bl_idname = "NODE_PT_HIG_Options"
    bl_label = "HIG Options"
    bl_region_type = "UI"
    bl_space_type = "NODE_EDITOR"
    bl_category = "Options"

    @classmethod
    def poll(cls, context):
        return context.space_data.node_tree != None

    def draw(self, context):
        if not context.scene.use_render:
            self.layout.operator("hig.override_use_render")
        else: 
            for prop in context.scene.use_render:
                if prop.n_tree == context.space_data.node_tree:
                    break
            else:
                self.layout.operator("hig.override_use_render")

class NODE_PT_HIG_Options_override(bpy.types.Panel):
    bl_parent_id = "NODE_PT_HIG_Options"
    bl_label = ""
    bl_region_type = "UI"
    bl_space_type = "NODE_EDITOR"

    @classmethod
    def poll(cls, context):
        if context.scene.use_render:
            for prop in context.scene.use_render:
                if prop.n_tree == context.space_data.node_tree:
                    return True
            else:
                return False
    
    def draw_header(self,context):
        property_group = search_prop_group_by_ntree(self,context)
        self.layout.prop(property_group,"use_render")

    def draw(self, context):
        property_group = search_prop_group_by_ntree(self,context)
        self.layout.prop(property_group,"value")


def update_use_render(self,context):
    property_group = search_prop_group_by_ntree(self, context)
    override = property_group.value
    switch = property_group.use_render
    
    for node in context.space_data.node_tree.nodes:
        if "Show Render" in node.inputs:
            if switch:
                node['show_render_backup'] = node.inputs['Show Render'].default_value
            else:
                node.inputs['Show Render'].default_value = node['show_render_backup']
    update_value(self, context)


def update_value(self,context):
    property_group = search_prop_group_by_ntree(self, context)
    override = property_group.value
    switch = property_group.use_render
    
    for node in context.space_data.node_tree.nodes:
        if "Show Render" in node.inputs:
            if switch:
                node.inputs['Show Render'].default_value = override

class use_render_props(bpy.types.PropertyGroup):
    use_render: BoolProperty(name="Override Show Render", default=False,update=update_use_render)
    value: BoolProperty(name="Show Render", default=False,update=update_value)
    n_tree: PointerProperty(type=bpy.types.NodeTree)


def change_higgsas_nodes_presets():
    nodes_dir_path = Path(__file__).parent
    dir_name = nodes_dir_path.name
    # print(nodes_dir_path)
    # print(dir_name)
    app_lib_data = {}
    for lib in bpy.context.preferences.filepaths.asset_libraries:
        app_lib_data[lib.name] = lib.path

    asset_libraries = bpy.context.preferences.filepaths.asset_libraries
    if dir_name not in app_lib_data:
        bpy.ops.preferences.asset_library_add(directory=str(nodes_dir_path)+os.sep)
        asset_libraries[-1].name = dir_name
    else:
        asset_libraries.get(dir_name).path = dir_name

    app_lib_data.clear()

class Higgsas_Nodes_Prefs(bpy.types.AddonPreferences):
    bl_idname = __package__

    def draw(self, context):
        layout = self.layout
        col = layout.box().column(align=True)
        row = col.row()
        row.label(text="已经启用Higgsas节点资产,前往节点添加面板即可查看.")
        row = col.row()
        row.label(text="是否添加到资产库?(可在资产浏览器中查看节点缩略图,理解节点大意)")
        row = col.row().split(factor=0.8, align=False)
        row.operator("hig.add_higgsas_nodes_libassets")
        u = row.operator("wm.url_open", text="使用手册", icon="WORLD")
        u.url = "https://higgsas-geo-nodes-manual.readthedocs.io/en/latest/"
        row = col.row()
        row.label(text="别忘记保存您的设置!")
        


class Add_Higgsas_Nodes_LibAssets(bpy.types.Operator):
    bl_idname = "hig.add_higgsas_nodes_libassets"
    bl_label = "添加 Higgsas Nodes 到资产库"
    bl_description = "可在资产浏览器中查看节点缩略图,理解大意"
    def execute(self, context):
        change_higgsas_nodes_presets()
        return {'FINISHED'}
    
classes = (
    NODE_OT_group_add,
    # NODE_PT_HIG_Options,
    # override_use_render,
    # use_render_props,
    # NODE_PT_HIG_Options_override,
    Add_Higgsas_Nodes_LibAssets,
    Higgsas_Nodes_Prefs,
    )


def register():
    global geo_node_group_cache

    with open(os.path.join(os.path.dirname(__file__), "geometry_nodes.json"), 'r') as f:
        geo_node_group_cache = json.loads(f.read())

    if not hasattr(bpy.types, "NODE_MT_Higgsas_menu"):
        bpy.utils.register_class(NODE_MT_Higgsas_menu)
        bpy.types.NODE_MT_add.append(add_hig_button)
    for cls in classes:
        bpy.utils.register_class(cls)

    geo_cat_generator()
    # if not bpy.app.timers.is_registered(change_higgsas_nodes_presets):
    #     bpy.app.timers.register(change_higgsas_nodes_presets, first_interval=1)

    # bpy.types.Scene.use_render = bpy.props.CollectionProperty(
    #     type=use_render_props)

def unregister():
    if hasattr(bpy.types, "NODE_MT_Higgsas_menu"):
        bpy.types.NODE_MT_add.remove(add_hig_button)
    for cls in classes:
        bpy.utils.unregister_class(cls)

    # del bpy.types.Scene.use_render
    
    # if bpy.app.timers.is_registered(change_higgsas_nodes_presets):
    #     bpy.app.timers.unregister(change_higgsas_nodes_presets)

if __name__ == "__main__":
    register()
